@@ -1,18 +1,27 @@
-from quart import Quart, jsonify, Blueprint
-from quart.views import http_method_funcs
+"""pint.py
+
+Main file that provides the definitions for the Pint class and its dependants for enhancing the quart decorators
+and hooking into them in order to generate the openapi documentaion
+"""
+
 import json
-from jsonschema import Draft4Validator, RefResolver, FormatChecker
-from jsonschema.exceptions import ValidationError
 import logging
-from inspect import isclass
+from collections import namedtuple
 from http import HTTPStatus
+from inspect import isclass
+from typing import Any, Callable, Dict, Iterable, Optional, Tuple, Union
+
+from jsonschema import Draft4Validator, FormatChecker, RefResolver
+from jsonschema.exceptions import ValidationError
+from quart import Blueprint, Quart, jsonify
+from quart.views import http_method_funcs
+
 from .cors import crossdomain
 from .resource import Resource
-from .utils import cached_property, merge, camel_to_snake
-from .typing import ValidatorTypes, ExpectedDescList
-from typing import Optional, Union, Dict, Callable, Iterable, Tuple, Any
+from .typing import ExpectedDescList, ValidatorTypes
+from .utils import cached_property, camel_to_snake, merge
 
-logger = logging.getLogger('quart.serving')
+LOGGER = logging.getLogger('quart.serving')
 
 def _expand_params_desc(data: Dict[str, Any]) -> None:
     """Used to convert `{'param': 'Description String'}` into
@@ -25,15 +34,18 @@ def _expand_params_desc(data: Dict[str, Any]) -> None:
             if isinstance(description, str):
                 data['params'][name] = {'description': description}
 
-class BaseRest(object):
-    """Base object for handling the RESTful resources for routing and generating swagger. Used by :class:`Pint`
+class BaseRest():
+    """Base object for handling the RESTful resources for routing and generating openapi json. Used by :class:`Pint`
     and :class:`PintBlueprint`
     """
 
-    def __init__(self, *args: Any, title: Optional[str]=None, contact: Optional[str]=None,
-                 contact_url: Optional[str]=None, contact_email: Optional[str]=None,
-                 version: str='1.0', description: Optional[str]=None, validate: bool=True,
-                 base_model_schema: Optional[Union[str, Dict[str, Any], RefResolver]]=None,
+    ApiInfo = namedtuple('ApiInfo', ['title', 'description', 'version'])
+    ContactInfo = namedtuple('ContactInfo', ['name', 'url', 'email'])
+
+    def __init__(self, *args: Any, title: Optional[str] = None, contact: Optional[str] = None,
+                 contact_url: Optional[str] = None, contact_email: Optional[str] = None,
+                 version: str = '1.0', description: Optional[str] = None, validate: bool = True,
+                 base_model_schema: Optional[Union[str, Dict[str, Any], RefResolver]] = None,
                  **kwargs: Any) -> None:
         r"""Construct the Base Rest object
 
@@ -56,12 +68,11 @@ class BaseRest(object):
         self._validate = validate
         self._resources = []
         self._schema = None
-        self.title = title
-        self.description = description
-        self.version = version
-        self.contact = contact
-        self.contact_url = contact_url
-        self.contact_email = contact_email
+        self._info = BaseRest.ApiInfo(title, description, version)
+        # self.title = title
+        # self.description = description
+        # self.version = version
+        self._contact_obj = BaseRest.ContactInfo(contact, contact_url, contact_email)
         if base_model_schema is not None:
             if isinstance(base_model_schema, str):
                 with open(base_model_schema, 'r', encoding='utf-8') as f:
@@ -71,7 +82,37 @@ class BaseRest(object):
                 self._ref_resolver = RefResolver.from_schema(base_model_schema)
             elif isinstance(base_model_schema, RefResolver):
                 self._ref_resolver = base_model_schema
-        self.register_error_handler(ValidationError, self.handle_json_validation_exc)
+        self.register_error_handler(ValidationError, self.handle_json_validation_exc)  # pylint: disable=no-member
+
+    @property
+    def title(self) -> str:
+        """Return title string from API"""
+        return self._info.title
+
+    @property
+    def description(self) -> str:
+        """Return info description string from API"""
+        return self._info.description
+
+    @property
+    def version(self) -> str:
+        """Return version string from API"""
+        return self._info.version
+
+    @property
+    def contact(self) -> str:
+        """Return contact name string from API"""
+        return self._contact_obj.name
+
+    @property
+    def contact_url(self) -> str:
+        """Return contact url from API"""
+        return self._contact_obj.url
+
+    @property
+    def contact_email(self) -> str:
+        """Return contact info email from API"""
+        return self._contact_obj.email
 
     @staticmethod
     def handle_json_validation_exc(error: ValidationError) -> Dict[str, Union[str, Dict[str, str]]]:
@@ -88,7 +129,7 @@ class BaseRest(object):
            :meth:`quart.Quart.register_error_handler`
                Registering error handlers with quart
         """
-        logger.error('request body validation failed, returning error: msg: %s, instance: %r',
+        LOGGER.error('request body validation failed, returning error: msg: %s, instance: %r',
                      error.message, error.instance)
         return jsonify({
             'message': 'Request Body failed validation',
@@ -119,7 +160,7 @@ class BaseRest(object):
     def __schema__(self) -> Dict[str, Union[str, Dict[str, Any]]]:
         """The schema produced by the Swagger object using the information in this instance"""
         if not self._schema:
-            from .swagger import Swagger
+            from .swagger import Swagger # pylint: disable=import-outside-toplevel
             self._schema = Swagger(self).as_dict()
         return self._schema
 
@@ -132,8 +173,8 @@ class BaseRest(object):
         return self._validators[name] if name in self._validators else None
 
     def _add_resource(self, resource: Union[Resource, Callable],
-                     path: str, methods: Iterable[str], endpoint: Optional[str]=None, *args,
-                      provide_automatic_options: bool=True, **kwargs: Any) -> None:
+                      path: str, methods: Iterable[str], *args, endpoint: Optional[str] = None,
+                      provide_automatic_options: bool = True, **kwargs: Any) -> None:
         r"""Called by :meth:`route` in order to process the resource or view function and only add it to the
         list of openapi resources if it's a class, allowing paths to be left out of the openapi documentation
         by declaring them as functions.
@@ -155,11 +196,10 @@ class BaseRest(object):
                 setattr(resource, f'_pint_{resource.__name__}_view_wrapper', view_func)
             methods = list(resource.methods)
             self._resources.append((resource, path, methods))
-        super().add_url_rule(path, endpoint, view_func, methods,
-                          provide_automatic_options=provide_automatic_options, **kwargs)
+        super().add_url_rule(path, endpoint, view_func, methods,  # pylint: disable=no-member
+                             provide_automatic_options=provide_automatic_options, **kwargs)
 
-    def param(self, name: str, description: Optional[str]=None,
-              _in: str='query', **kwargs: Dict[str, Any]) -> Callable:
+    def param(self, name: str, description: Optional[str] = None, _in: str = 'query', **kwargs: Any) -> Callable:
         r"""Decorator for describing parameters for a given resource or specific request method.
 
         :param name: Parameter name in documention
@@ -198,8 +238,7 @@ class BaseRest(object):
         param['description'] = description
         return self.doc(params={name: param})
 
-    def response(self, code: HTTPStatus, description: str, validator: ValidatorTypes=None,
-                 **kwargs: Dict[str, Any]) -> Callable:
+    def response(self, code: HTTPStatus, description: str, validator: ValidatorTypes = None, **kwargs: Any) -> Callable:
         r"""Decorator for documenting the response from a route
 
         :param code: The HTTP Response code for this response
@@ -226,7 +265,7 @@ class BaseRest(object):
         """
         return self.doc(responses={code: (description, validator, kwargs)})
 
-    def route(self, path: str, methods: Iterable[str]=['GET'], *args, **kwargs) -> Callable:
+    def route(self, path: str, *args, methods: Optional[Iterable[str]] = None, **kwargs) -> Callable:
         r"""Decorator for establishing routes
 
         :param path: the path to route on, should start with a `/`, may contain params converters
@@ -238,6 +277,8 @@ class BaseRest(object):
 
         .. seealso:: Base class version :meth:`~quart.Quart.route`
         """
+        if methods is None:
+            methods = ['GET']
         def decorator(func_or_viewcls: Union[Resource, Callable]) -> Union[Resource, Callable]:
             doc = kwargs.pop('doc', None)
             if doc is not None:
@@ -263,7 +304,8 @@ class BaseRest(object):
 
         .. todo:: Ensure that models with the same name in different categories don't conflict
         """
-        validator = Draft4Validator({ '$ref': f'#/components/{category}/{name}' }, resolver=self._ref_resolver, format_checker=FormatChecker())
+        validator = Draft4Validator({'$ref': f'#/components/{category}/{name}'},
+                                    resolver=self._ref_resolver, format_checker=FormatChecker())
         self._validators[name] = validator
         return validator
 
@@ -285,7 +327,8 @@ class BaseRest(object):
         self._validators[name] = validator
         return validator
 
-    def _handle_doc(self, cls: Callable, doc: Dict[str, Any]) -> None:
+    @staticmethod
+    def _handle_doc(documented: Callable, doc: Dict[str, Any]) -> None:
         """Internal function for merging doc specs for various HTTP verbs and handling expects"""
         # adapted from flask_restplus
         _expand_params_desc(doc)
@@ -296,9 +339,9 @@ class BaseRest(object):
                 _expand_params_desc(doc[http_method])
                 if 'expect' in doc[http_method] and not isinstance(doc[http_method]['expect'], (list, tuple)):
                     doc[http_method]['expect'] = [doc[http_method]['expect']]
-        cls.__apidoc__ = merge(getattr(cls, '__apidoc__', {}), doc)
+        documented.__apidoc__ = merge(getattr(documented, '__apidoc__', {}), doc)
 
-    def doc(self, **kwargs: Dict[str, Any]) -> Callable:
+    def doc(self, **kwargs: Any) -> Callable:
         """Generic decorator for adding docs via keys pointing to dictionaries
 
         Examples:
@@ -419,11 +462,32 @@ class Pint(BaseRest, Quart):
     route.
     """
 
-    def __init__(self, *args, **kwargs) -> None:
-        """ Construct the Pint object, see :meth:`BaseRest.__init__` for an explanation of the args and kwargs."""
+    def __init__(self, *args, no_openapi: bool = False, **kwargs) -> None:
+        r"""Construct the Pint object, see :meth:`BaseRest.__init__` for an explanation of the args and kwargs.
+
+        :param \*args: non-keyword args passed to :class:`BaseRest`
+        :param no_openapi: set this to True to disable the auto creation of the /openapi.json route
+                           in order to allow custom manipulation of the route
+        :param \*\*kwargs: keyword args passed to :class:`BaseRest`
+
+        If you pass `no_openapi=True` then you can customize the openapi route by creating it yourself:
+
+        .. code-block:: python
+
+            app = Pint('Custom', no_openapi=True)
+
+            @app.route('/openapi.json')
+            # add other decorators if desired
+            async def openapi():
+              # add other logic if desired
+              return jsonify(app.__schema__)
+
+        """
         super().__init__(*args, **kwargs)
         self.config['JSON_SORT_KEYS'] = False
-        self.add_url_rule('/openapi.json', 'openapi', SwaggerView.as_view('swaggerview', self), ['GET', 'OPTIONS'])
+        if not no_openapi:
+            self.add_url_rule('/openapi.json', 'openapi',
+                              OpenApiView.as_view('openapi', self), ['GET', 'OPTIONS'])
 
 class PintBlueprint(BaseRest, Blueprint):
     """Use this instead of :class:`quart.Blueprint` objects to allow using Resource class objects with them"""
@@ -432,15 +496,16 @@ class PintBlueprint(BaseRest, Blueprint):
         """Will forward all arguments and keyword arguments to Blueprints"""
         super().__init__(*args, **kwargs)
 
-    def register(self, app: Pint, first_registration: bool, *, url_prefix: Optional[str]=None) -> None:
+    # pylint: disable=arguments-differ
+    def register(self, app: Pint, first_registration: bool, *, url_prefix: Optional[str] = None, **kwargs) -> None:
         """override the base :meth:`~quart.Blueprint.register` method to add the resources to the app registering
         this blueprint, then call the parent register method
         """
         prefix = url_prefix or self.url_prefix or ''
-        app._resources.extend([(res, f'{prefix}{path}', methods) for res, path, methods in self._resources])
-        super().register(app, first_registration, url_prefix=url_prefix)
+        app.resources.extend([(res, f'{prefix}{path}', methods) for res, path, methods in self._resources])
+        super().register(app, first_registration, url_prefix=url_prefix, **kwargs)
 
-class SwaggerView(Resource):
+class OpenApiView(Resource):
     """The :class:`Resource` used for the '/openapi.json' route
 
     It also uses CORS to set the Access-Control-Allow-Origin header to "*" for this route so that the
@@ -449,7 +514,7 @@ class SwaggerView(Resource):
     .. todo:: Allow customizing the origin for CORS on the openapi.json
     """
     def __init__(self, api: Pint) -> None:
-        """Construct the SwaggerView
+        """Construct the OpenApiView
 
         :param api: will be an instance of a :class:`Pint` object, the :attr:`~Pint.__schema__`
                     property will be returned for `get` requests to '/openapi.json'
@@ -460,4 +525,5 @@ class SwaggerView(Resource):
     # this way it can also be used with swagger UI
     @crossdomain(origin='*')
     async def get(self):
+        """Return the schema for get requests"""
         return jsonify(self.api.__schema__), 200
